<?php

/**
 * nicedit.admin.inc
 *
 * This page is only loaded when doing admin stuff.
 * Which minimizes the number of lines that get parsed on most pages.
 */

/**
 * Implement admin/settings/nicedit.
 */
function nicedit_settings_form() {
  $nesid = $_REQUEST['nesid'];
  if (isset($nesid)) {
    if ($nesid <= 0) {
      drupal_set_message(t('Creating a new configuration'));
    }
    else {
      $setting = _nicedit_load_setting($nesid);
      drupal_set_message(t('Configuring settings for !name', array('!name' => ucfirst($setting->name))));
    }
  }
  else {
    $setting = _nicedit_load_setting('default');
  }
  $form = _nicedit_settings_form($setting, isset($nesid) && $nesid == 0);
  $form['#submit'][] = 'nicedit_settings_form_submit';
  return system_settings_form($form);
}

function _nicedit_settings_form($setting = NULL, $new = FALSE) {
  drupal_add_js(drupal_get_path('module', 'nicedit') .'/nicedit-admin.js');
  $form = array();
  if ($new || (isset($setting) && $setting->name != 'default')) {
    $form['nicedit_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#default_value' => isset($setting->name) ? $setting->name : '',
      '#description' => t('Name the configuration, used when selecting which configuration to use.'),
      '#required' => TRUE,
    );
  }
  $form['nicedit_fullpanel'] = array(
    '#type' => 'checkbox',
    '#title' => t('Full Panel'),
    '#default_value' => isset($setting->fullpanel) ? $setting->fullpanel : 0,
    '#description' => t('Selecting this, selects all the other buttons'),
  );
  $buttons = _nicedit_buttons();
  // NOTE: this index can't be 'buttons' because that conflicts with the settings form buttons
  $form['nicedit_buttons'] = array(
    '#type' => 'checkboxes',
    '#options' => $buttons,
    '#default_value' => isset($setting->buttons) ? $setting->buttons : array_keys($buttons),
    '#theme' => 'nicedit_settings_form_buttons',
    '#description' => t('Select individual buttons to display'),
  );
  if (isset($setting->nesid)) {
    $form['nesid'] = array(
      '#type' => 'hidden',
      '#value' => $setting->nesid,
    );
  }
  if (isset($_REQUEST['name'])) {
    $form['name'] = array(
      '#type' => 'hidden',
      '#value' => $_REQUEST['name'],
    );
  }
  if (isset($_REQUEST['page'])) {
    $form['page'] = array(
      '#type' => 'hidden',
      '#value' => $_REQUEST['page'],
    );
  }
  return $form;
}

function nicedit_settings_form_submit($form, &$form_state) {
  $nesid = $form_state['values']['nesid'];
  if ($form_state['values']['op'] == t('Save configuration')) {
    $fullpanel = $form_state['values']['nicedit_fullpanel'];
    $buttons = serialize($form_state['values']['nicedit_buttons']);
    if (isset($nesid) && $nesid > 0) {
      db_query("UPDATE {nicedit_settings} SET fullpanel = %d, buttons = '%s' WHERE nesid = %d", $fullpanel, $buttons, $nesid);
    }
    else {
      $name = isset($form_state['values']['nicedit_name']) ? $form_state['values']['nicedit_name'] : 'default';
      db_query("INSERT INTO {nicedit_settings} (fullpanel, buttons, name) VALUES (%d, '%s', '%s')", $fullpanel, $buttons, $name);
    }
  }
  elseif ($form_state['values']['op'] == t('Reset to defaults') && isset($nesid) && $nesid > 0) {
    db_query("DELETE FROM {nicedit_settings} WHERE nesid = %d", $nesid);
    drupal_set_message(t('The configuration options have been reset to their default values.'));
  }
  if (isset($form_state['values']['page'])) {
    if (isset($form_state['values']['name'])) {
      $form_state['values']['nesid'] = $nesid;
      _nicedit_config_form_submit_save($form_state['values']);
    }
    drupal_goto($form_state['values']['page']);
  }
}

function theme_nicedit_settings_form_buttons($form) {
  $rows = array();
  foreach ($form as $element_id => $element) {
    if ($element_id[0] != '#') {
      if (!isset($cols)) {
        $cols = array();
      }
      $cols[] = drupal_render($element);
      if (count($cols) == 3) {
        $rows[] = $cols;
        unset($cols);
      }
    }
  }
  if (isset($cols)) {
    $rows[] = $cols;
  }
  return theme('table', array(), $rows);
}

/**
 * Implement admin/settings/config.
 */
function nicedit_config_form() {
  // get the arguments
  $page = $_REQUEST['page'];
  $name = $_REQUEST['name'];
  if ($page && $name) {
    drupal_set_message(t('Configuring field %name on page %page.', array('%name' => $name, '%page' => $page)));
  }
  else {
    drupal_set_message(t('The field or page name were lost.'), 'error');
    drupal_goto('admin/settings/nicedit');
  }

  // load the existing settings configurations
  $results = db_query("SELECT * FROM {nicedit_settings} ORDER BY nesid ASC");
  $settings = array();
  while ($setting = db_fetch_object($results)) {
    if ($setting->name == 'default') {
      $default_nesid = $setting->nesid;
    }
    $settings[$setting->nesid] = ucfirst($setting->name);
  }
  if (count($settings) == 0) {
    $settings[-1] = 'default';
  }
  if ($page && $name) {
    if ($config_nesid = db_result(db_query("SELECT nesid FROM {nicedit_config} WHERE page = '%s' AND name = '%s'", $page, $name))) {
      $nesid = $config_nesid;
    }
  }
  if (!isset($nesid) && isset($default_nesid)) {
    $nesid = $default_nesid;
  }

  // Let the user select an existing settings configuration
  $form = array();
  $form['nesid'] = array(
    '#type' => 'select',
    '#options' => $settings,
    '#default_value' => $nesid,
    '#title' => t('Settings configuration'),
    '#description' => t('Select a settings configuration to use with this field.  Try to re-use existing settings configurations whenever possible.'),
  );

  // save the page and name
  $form['page'] = array(
    '#type' => 'hidden',
    '#value' => $page,
  );
  $form['name'] = array(
    '#type' => 'hidden',
    '#value' => $name,
  );

  // Add the buttons
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );
  $form['edit'] = array(
    '#type' => 'submit',
    '#value' => t('Edit settings'),
  );
  $form['new'] = array(
    '#type' => 'submit',
    '#value' => t('Create new settings'),
  );
  $form['disable'] = array(
    '#type' => 'submit',
    '#value' => t('Disable editor on this field'),
  );
  return $form;
}

function nicedit_config_form_submit($form, &$form_state) {
  if ($form_state['values']['op'] == t('Save configuration')) {
    _nicedit_config_form_submit_save($form_state['values'], TRUE);
    drupal_goto($form_state['values']['page']);
  }
  elseif ($form_state['values']['op'] == t('Edit settings')) {
    _nicedit_config_form_submit_save($form_state['values']);
    $query = array();
    $query[] = "nesid=". $form_state['values']['nesid'];
    $query[] = "page=". $form_state['values']['page'];
    $query[] = "name=". $form_state['values']['name'];
    drupal_goto('admin/settings/nicedit', implode('&', $query));
  }
  elseif ($form_state['values']['op'] == t('Disable editor on this field')) {
    $query = array();
    $query[] = "name=". $form_state['values']['name'];
    $query[] = "page=". $form_state['values']['page'];
    $query[] = "enable=0";
    drupal_goto('admin/settings/nicedit/enable', implode('&', $query));
  }
  elseif ($form_state['values']['op'] == t('Create new settings')) {
    $query = array();
    $query[] = "nesid=0";
    $query[] = "page=". $form_state['values']['page'];
    $query[] = "name=". $form_state['values']['name'];
    drupal_goto('admin/settings/nicedit', implode('&', $query));
  }
}

function _nicedit_config_form_submit_save($form_values, $message = FALSE) {
  $page = $form_values['page'];
  $name = $form_values['name'];
  $default_nesid = db_result(db_query("SELECT nesid FROM {nicedit_settings} WHERE name = 'default'"));
  if ($form_values['nesid'] == $default_nesid) {
    db_query("DELETE FROM {nicedit_config} WHERE page = '%s' AND name = '%s'", $page, $name);
  }
  elseif (db_result(db_query("SELECT page FROM {nicedit_config} WHERE page = '%s' AND name = '%s'", $page, $name))) {
    db_query("UPDATE {nicedit_config} SET nesid = %d WHERE page = '%s' AND name = '%s'", $form_values['nesid'], $page, $name);
  }
  else {
    db_query("INSERT INTO {nicedit_config} (nesid, page, name, disabled) VALUES (%d, '%s', '%s', 0)", $form_values['nesid'], $page, $name);
  }
  if ($message) {
    drupal_set_message(t('The configuration options have been saved.'));
  }
}

/**
 * Implement admin/settings/nicedit/enable.
 */
function nicedit_settings_enable_page() {
  $page = $_REQUEST['page'];
  $name = $_REQUEST['name'];
  $disabled = isset($_REQUEST['enable']) ? ($_REQUEST['enable'] == 1 ? 0 : 1) : 0;
  $config = db_fetch_object(db_query("SELECT page, nesid FROM {nicedit_config} WHERE page = '%s' AND name = '%s'", $page, $name));
  if (isset($config->page)) {
    if ($disabled || isset($config->nesid)) {
      db_query("UPDATE {nicedit_config} SET disabled = %d WHERE page = '%s' AND name = '%s'", $disabled, $page, $name);
    }
    else {
      db_query("DELETE FROM {nicedit_config} WHERE page = '%s' AND name = '%s'", $page, $name);
    }
  }
  elseif ($disabled) {
    db_query("INSERT INTO {nicedit_config} (page, name, disabled) VALUES ('%s', '%s', %d)", $page, $name, $disabled);
  }
  drupal_goto($page);
}

